use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
925043
API
Andrew
AnyParser
AutoLink
BalancedEvent
BlockParser
Bucheli
CLI
CaptureEvents
CapturedEvents
CheckedOutput
CodeBlock
DROLSKY
DROLSKY's
Dave
DebugPrinter
Denis
Dialect
Document
EndBlockquote
EndCode
EndDocument
EndEmphasis
EndHTMLTag
EndHeader
EndLink
EndListItem
EndOrderedList
EndParagraph
EndStrikethrough
EndStrong
EndTable
EndTableBody
EndTableCell
EndTableHeader
EndTableRow
EndUnorderedList
Etheridge
Event
EventAsText
EventsAsMethods
FilterHandler
Fish
Fragment
GitHub
Gruber
Gruber's
HTMLBlock
HTMLComment
HTMLCommentBlock
HTMLEntity
HTMLFilter
HTMLStream
HTMLTag
Handler
HorizontalRule
Hornburg
Hukins
Ibaev
Image
Internal
Jason
Jonas
Karen
Konrad
LineBreak
MARKDENT
Manual
Markdent
Markdent's
Markdown
McIntosh
MinimalTree
Multiplexer
Null
Parser
PayPal
Polina
Preformatted
Racke
Regexes
Role
Rolsky
Rolsky's
SHAs
Shlomi
Shubina
Simple
Smedegaard
SpanParser
Speer
Standard
StartBlockquote
StartCode
StartDocument
StartEmphasis
StartHTMLTag
StartHeader
StartLink
StartListItem
StartOrderedList
StartParagraph
StartStrikethrough
StartStrong
StartTable
StartTableBody
StartTableCell
StartTableHeader
StartTableRow
StartUnorderedList
Stefan
Text
Theory
Tom
Types
XML's
andrew
autarch
bin
blockquote
blockquotes
colspan
dionys
doctype
dr
drolsky
escapeable
esque
ether
html
inline
jmac
konrad
lang
lib
markdent
mdtest
namespace
parameterized
parser's
parsers
preformatted
racke
shlomif
strikethrough
tom
unordered
uri
wiki
